﻿using System;
using System.ComponentModel;
using System.Windows.Input;

using Microsoft.Win32;

using XmlViewer.BusinessLogic;
using System.Windows.Data;

namespace XmlViewer.ViewModels
{
    /// <summary>
    /// View Model for the main view.
    /// </summary>
    public class MainWindowViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Returns the Window display name.
        /// </summary>
        private readonly string displayName;
        public string DisplayName
        {
            get { return this.displayName; }
            set {;}
        }

        /// <summary>
        /// Gets or sets the XML contents to be displayed.
        /// </summary>
        private string xmlContents;
        public string XmlContents
        {
            get { return this.xmlContents; }
            set
            {
                this.xmlContents = value;
                RaisePropertyChanged("XmlContents");
            }
        }

        /// <summary>
        /// Displays an open dialog to prompt the user for the XML file
        /// to be displayed.
        /// </summary>
        private ICommand displayOpenDialog;
        public ICommand DisplayOpenDialog
        {
            get { return displayOpenDialog; }
            set
            {
                this.displayOpenDialog = value;
                RaisePropertyChanged("DisplayOpenDialog");
            }
        }

        /// <summary>
        /// Displays the about page in the Window.
        /// </summary>
        private ICommand displayAboutPage;
        public ICommand DisplayAboutPage
        {
            get { return displayAboutPage; }
            set
            {
                this.displayAboutPage = value;
                RaisePropertyChanged("DisplayAboutPage");
            }
        }

        /// <summary>
        /// TOOD: this should go into a base class so that all views have the
        /// ability to close the application. See below for comments in 
        /// CloseApp on the messaging system.
        /// </summary>
        private ICommand closeApplication;
        public ICommand CloseApplication
        {
            get { return closeApplication; }
            set
            {
                this.closeApplication = value;
                RaisePropertyChanged("CloseApplication");
            }
        }

        /// <summary>
        /// Constructs a new instance of MainWindowViewModel.
        /// </summary>
        public MainWindowViewModel()
        {
            this.displayName = "Xml Viewer";
            displayOpenDialog = new ActionCommand(e => XmlContents = DisplayOpenPrompt());
            displayAboutPage = new ActionCommand(e => DisplayAboutPrompt());
            closeApplication = new ActionCommand(e => CloseApp());
        }

        /// <summary>
        /// TODO:
        /// This should just display an open dialog otherwise it violates the
        /// "do one thing only rule". At the moment it returns the contents
        /// of the file.
        /// </summary>
        /// <returns>The contents of the file.</returns>
        private string DisplayOpenPrompt()
        {
            // TODO:
            // Create a WPF open dialog that is displayed inline with the main
            // window.
            // Configure open file dialog box
            OpenFileDialog openDialog = new OpenFileDialog();
            openDialog.FileName   = "Document";
            openDialog.DefaultExt = ".xml";
            openDialog.Filter     = "XML documents (.xml)|*.xml";

            // Show open file dialog box
            Nullable<bool> result = openDialog.ShowDialog();

            // Process open file dialog box results
            if (result == true)
            {
                // TODO:
                // Need to use dependency injection on the file handler class
                // so as not to create a dependency on concrete class FileHandler.
                IFileHandler fileHandler = new FileHandler();
                return fileHandler.Contents(openDialog.FileName);
            }

            return null;
        }

        /// <summary>
        /// 
        /// </summary>
        private void DisplayAboutPrompt()
        {
//             AboutViewModel aboutPage = new AboutViewModel();
//             ICollectionView collectionView = CollectionViewSource.GetDefaultView();
//             if (collectionView != null)
//                 collectionView.MoveCurrentTo();
        }

        /// <summary>
        /// 
        /// </summary>
        private void CloseApp()
        {
            // TODO: setup a messaging system to call the application close.
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Fires the property changed event when a property on this class is
        /// changed.
        /// </summary>
        /// <param name="propertyName">Name of property changed.</param>
        public void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion
    }
}
